@@ -75,6 +75,7 @@ import com.amazonaws.regions.Regions;
 import com.amazonaws.services.s3.AmazonS3Client;
 import com.amazonaws.services.s3.AmazonS3EncryptionClient;
 import com.amazonaws.services.s3.S3ClientOptions;
+import com.amazonaws.services.s3.model.AmazonS3Exception;
 import com.amazonaws.services.s3.model.CryptoConfiguration;
 import com.amazonaws.services.s3.model.EncryptionMaterials;
 import com.amazonaws.services.s3.model.Region;
@@ -135,6 +136,7 @@ public class S3FileStorageFactory implements FileStorageProvider {
 
     @Override
     public S3FileStorage getFileStorage(URI uri) throws OXException {
+        try{
         S3FileStorage storage = storages.get(uri);
         if (null == storage) {
             LOG.debug("Initializing S3 client for {}", uri);
@@ -146,16 +148,25 @@ public class S3FileStorageFactory implements FileStorageProvider {
             /*
              * create client
              */
-            AmazonS3Client client = initClient(filestoreID);
+            AmazonS3ClientInfo clientInfo = initClient(filestoreID);
+            AmazonS3Client client = clientInfo.client;
             String bucketName = initBucket(client, filestoreID);
             LOG.debug("Using \"{}\" as bucket name.", bucketName);
-            S3FileStorage newStorage = new S3FileStorage(client, bucketName, extractFilestorePrefix(uri));
+            S3FileStorage newStorage = new S3FileStorage(client, clientInfo.encrypted, bucketName, extractFilestorePrefix(uri));
             storage = storages.putIfAbsent(uri, newStorage);
             if (null == storage) {
                 storage = newStorage;
             }
         }
-        return storage;
+            return storage;
+        }catch(OXException ex){
+            if(ex.getCause() instanceof AmazonS3Exception && ((AmazonS3Exception)ex.getCause()).getStatusCode() == 400){
+                // throw a simple OXException here to be able to forward this exception to rmi clients (Bug #42102)
+                throw new OXException(S3ExceptionCode.BadRequest.getNumber(), S3ExceptionMessages.BadRequest_MSG);
+            }else{
+                throw ex;
+            }
+        }
     }
 
     @Override
@@ -180,7 +191,7 @@ public class S3FileStorageFactory implements FileStorageProvider {
      * @return The client
      * @throws OXException
      */
-    private AmazonS3Client initClient(String filestoreID) throws OXException {
+    private AmazonS3ClientInfo initClient(String filestoreID) throws OXException {
         /*
          * prepare credentials
          */
@@ -193,17 +204,19 @@ public class S3FileStorageFactory implements FileStorageProvider {
         ClientConfiguration clientConfiguration = getClientConfiguration(filestoreID);
         AmazonS3Client client = null;
         String encryption = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".encryption", "none");
+        boolean encrypted;
         if (Strings.isEmpty(encryption) || "none".equals(encryption)) {
             /*
              * use default AmazonS3Client
              */
             client = new AmazonS3Client(credentials, clientConfiguration);
+            encrypted = false;
         } else {
             /*
              * use AmazonS3EncryptionClient
              */
-            client = new AmazonS3EncryptionClient(
-                credentials, getEncryptionMaterials(filestoreID, encryption), clientConfiguration, new CryptoConfiguration());
+            client = new AmazonS3EncryptionClient(credentials, getEncryptionMaterials(filestoreID, encryption), clientConfiguration, new CryptoConfiguration());
+            encrypted = true;
         }
         /*
          * configure client
@@ -224,7 +237,7 @@ public class S3FileStorageFactory implements FileStorageProvider {
         }
 
 
-        return client;
+        return new AmazonS3ClientInfo(client, encrypted);
     }
 
     private ClientConfiguration getClientConfiguration(String filestoreID) {
@@ -390,4 +403,19 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return authority;
     }
 
+    private static class AmazonS3ClientInfo {
+
+        /** The Amazon S3 client reference */
+        final AmazonS3Client client;
+
+        /** Whether associated Amazon S3 client reference has encryption enabled */
+        final boolean encrypted;
+
+        AmazonS3ClientInfo(AmazonS3Client client, boolean encrypted) {
+            super();
+            this.client = client;
+            this.encrypted = encrypted;
+        }
+    }
+
 }
